/*
 * Copyright (c) 2009 Jan Friesse <honzaf@users.sourceforge.net>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL ISC BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include "lwtt_parser.h"

struct XMLParserUserData {
  int status;
  unsigned int item_no;
  char type;
  GKeyFile *app_config;
};

float atof_C (const char *text)
{
  double num;
  int neg_mul;
  int i;
  double mul_by;

  num = 0;
  mul_by = 0.1;
  neg_mul = 1;

  for (i = 0; (i < strlen (text)) && (text[i] == ' ' || text[i] == '\t' || text[i] == '\n' || text[i] == '\r'); i++) ;

  if (i < strlen (text) && text[i] == '-') {
    neg_mul = -1;
    i++;
  }


  for (; i < strlen (text) && text[i] >= '0' && text[i] <= '9'; i++)
    num = num * 10 + (text[i] - '0');

  if (i < strlen (text) && text[i] == '.') {
    i++;
    for (; i < strlen (text) && text[i] >= '0' && text [i] <= '9'; i++) {
      num = num + (mul_by * (text[i] - '0'));
      mul_by /= 10;
    }
  }

  return (neg_mul * num);
}

static void parse_lwtt_start_element(GMarkupParseContext *context, const gchar *element_name, const gchar **attribute_names,
  const gchar **attribute_values, gpointer user_data, GError **error)
{
  struct XMLParserUserData *data = (struct XMLParserUserData *)user_data;
  const gchar **attr_name_iter;
  const gchar **attr_value_iter;
  char *dot_pos;
  unsigned int item_no;

  if (data->status == 0 && strcmp (element_name, "properties") == 0) {
    /* We found properties */
    data->status = 1;
  }

  if (data->status == 1 && strcmp (element_name, "entry") == 0) {
    /* We found entry*/
    for (attr_name_iter = attribute_names, attr_value_iter = attribute_values; *attr_name_iter; attr_name_iter++, attr_value_iter++) {
      if (strcmp (*attr_name_iter, "key") == 0) {
        /* And has attr key */
        dot_pos = strchr (*attr_value_iter, '.');

        if (dot_pos == NULL)
          continue;

        dot_pos++;
        data->type = 0;

        if (strcmp (dot_pos, "price") == 0) data->type = 'p';
        if (strcmp (dot_pos, "name") == 0) data->type = 'n';
        if (strcmp (dot_pos, "consumption") == 0) data->type = 'c';
        if (strcmp (dot_pos, "size.h") == 0) data->type = 'h';
        if (strcmp (dot_pos, "size.w") == 0) data->type = 'w';
        if (strcmp (dot_pos, "location.x") == 0) data->type = 'x';
        if (strcmp (dot_pos, "location.y") == 0) data->type = 'y';

        if (data->type == 0)
          continue;

        if (data->type == 'w' || data->type == 'h' || data->type == 'x' || data->type == 'y') {
          data->status = 2;

          continue;
        }

        /* Item no */
        if (!((*attr_value_iter)[0] >= '0' && (*attr_value_iter)[0] <= '9'))
          continue;

        item_no = atoi (*attr_value_iter);

        data->item_no = item_no;

        data->status = 2;
      }
    }
  }
}

static void parse_lwtt_end_element (GMarkupParseContext *context, const gchar *element_name, gpointer user_data, GError **error)
{
  struct XMLParserUserData *data = (struct XMLParserUserData *)user_data;

  if (data->status == 2 && strcmp (element_name, "entry") == 0) {
    data->status = 1;
  }

  if (data->status == 1 && strcmp (element_name, "properties") == 0) {
    data->status = 0;
  }
}

static void parse_lwtt_text (GMarkupParseContext *context, const gchar *text, gsize text_len, gpointer user_data, GError **error)
{
  struct XMLParserUserData *data = (struct XMLParserUserData *)user_data;
  char group_name[256];
  char tmp_text[256];

  if (data->status == 2) {
    /* Text longer then buffer -> ignore */
    if (text_len > sizeof (tmp_text) - 1)
      return;

    memcpy (tmp_text, text, text_len);
    tmp_text[text_len] = 0;

    sprintf (group_name, "task%d", data->item_no);

    if (data->type == 'n') {
      g_key_file_set_string (data->app_config, group_name, "name", tmp_text);
    }

    if (data->type == 'c') {
      g_key_file_set_integer (data->app_config, group_name, "consumption", atoi (tmp_text) / 60000);
    }

    if (data->type == 'p') {
      g_key_file_set_integer (data->app_config, group_name, "price", (int)(atof_C (tmp_text) * 1000.0));
    }

    if (data->type == 'x')
      g_key_file_set_integer (data->app_config, "mainwindow", "x", atoi (tmp_text));

    if (data->type == 'y')
      g_key_file_set_integer (data->app_config, "mainwindow", "y", atoi (tmp_text));

    if (data->type == 'w')
      g_key_file_set_integer (data->app_config, "mainwindow", "width", atoi (tmp_text));

    if (data->type == 'h')
      g_key_file_set_integer (data->app_config, "mainwindow", "height", atoi (tmp_text));
  }
}

gboolean parse_lwtt_file (const char *fname, GKeyFile **app_config, GError **error)
{
  FILE *f;
  char buf[256];
  size_t readed;
  GMarkupParseContext *parse_context;
  GMarkupParser parser;
  struct XMLParserUserData user_data;

  f = fopen (fname, "rt");
  if (!f) {
    g_set_error (error, G_FILE_ERROR, G_FILE_ERROR_ACCES, "Failed to open file: %s", fname);
    return FALSE;
  }

  memset (&parser, 0, sizeof (parser));
  parser.start_element = parse_lwtt_start_element;
  parser.end_element = parse_lwtt_end_element;
  parser.text = parse_lwtt_text;

  user_data.status = 0;
  user_data.app_config = g_key_file_new ();

  parse_context = g_markup_parse_context_new (&parser, 0, &user_data, NULL);

  while ((readed = fread (buf, 1, sizeof (buf), f)) != 0) {
    if (!g_markup_parse_context_parse (parse_context, buf, readed, error)) {
      g_key_file_free (user_data.app_config);
      return FALSE;
    }
  }

  g_markup_parse_context_free (parse_context);

  *app_config = user_data.app_config;

  return TRUE;
}
